@@ -62,17 +62,20 @@ from django.views.decorators.csrf import csrf_exempt
 from django.views.generic.base import View
 
 from .. import ca_settings
+from ..constants import REASON_CODES
 from ..extensions import SubjectAlternativeName
 from ..models import AcmeAccount
 from ..models import AcmeAuthorization
 from ..models import AcmeCertificate
 from ..models import AcmeChallenge
 from ..models import AcmeOrder
+from ..models import Certificate
 from ..models import CertificateAuthority
 from ..tasks import acme_issue_certificate
 from ..tasks import acme_validate_challenge
 from ..tasks import run_task
 from ..utils import check_name
+from ..utils import int_to_hex
 from ..utils import make_naive
 from ..utils import validate_email
 from .errors import AcmeBadCSR
@@ -99,7 +102,7 @@ from .responses import AcmeResponseUnsupportedMediaType
 from .utils import parse_acme_csr
 
 log = logging.getLogger(__name__)
-MessageTypeVar = TypeVar("MessageTypeVar", bound=jose.JSONObjectWithFields)
+MessageTypeVar = TypeVar("MessageTypeVar", bound=jose.json_util.JSONObjectWithFields)
 DirectoryMetaAlias = Dict[str, Union[str, List[str]]]
 
 
@@ -181,7 +184,7 @@ class AcmeDirectory(View):
 
         # Get some random data into the directory view, as explained in the Let's Encrypt directory:
         #   https://community.letsencrypt.org/t/adding-random-entries-to-the-directory/33417
-        rnd = jose.encode_b64jose(secrets.token_bytes(16))
+        rnd = jose.json_util.encode_b64jose(secrets.token_bytes(16))
 
         directory: Dict[str, Union[str, DirectoryMetaAlias]] = {
             rnd: "https://community.letsencrypt.org/t/adding-random-entries-to-the-directory/33417",
@@ -189,7 +192,7 @@ class AcmeDirectory(View):
             "newAccount": self._url(request, "acme-new-account", ca),
             "newNonce": self._url(request, "acme-new-nonce", ca),
             "newOrder": self._url(request, "acme-new-order", ca),
-            "revokeCert": "http://localhost:8000/django_ca/acme/todo/revoke-cert",
+            "revokeCert": self._url(request, "acme-revoke", ca),
         }
 
         # Construct a "meta" object if and add it if any fields are defined. Note that the meta object is
@@ -217,20 +220,22 @@ class AcmeGetNonceViewMixin:
     nonce_length = 32
     """Length of generated Nonces."""
 
+    def get_cache_key(self, nonce: str) -> str:
+        """Get the cache key for the given request and nonce."""
+        return f"acme-nonce-{self.kwargs['serial']}-{nonce}"
+
     def get_nonce(self) -> str:
         """Get a random Nonce and add it to the cache."""
 
         data = secrets.token_bytes(self.nonce_length)
-        nonce = jose.encode_b64jose(data)
-        cache_key = f"acme-nonce-{self.kwargs['serial']}-{nonce}"
-        cache.set(cache_key, 0)
+        nonce = jose.json_util.encode_b64jose(data)
+        cache.set(self.get_cache_key(nonce), 0)
         return nonce
 
     def validate_nonce(self, nonce: str) -> bool:
         """Validate that the given nonce was issued and was not used before."""
-        cache_key = f"acme-nonce-{self.kwargs['serial']}-{nonce}"
         try:
-            count = cache.incr(cache_key)
+            count = cache.incr(self.get_cache_key(nonce))
         except ValueError:
             # raised if cache_key is not set
             return False
@@ -247,7 +252,8 @@ class AcmeBaseView(AcmeGetNonceViewMixin, View, metaclass=abc.ABCMeta):
     """Base class for all ACME views."""
 
     requires_key = False  # True if we require a full key (-> new accounts)
-    jwk: jose.JWK
+    accepts_kid_or_jwk = False  # Set to true to accept both KID or JWK
+    jwk: jose.jwk.JWK
     jws: acme.jws.JWS
 
     @abc.abstractmethod
@@ -330,7 +336,7 @@ class AcmeBaseView(AcmeGetNonceViewMixin, View, metaclass=abc.ABCMeta):
 
         try:
             self.jws = acme.jws.JWS.json_loads(request.body)
-        except (jose.DeserializationError, TypeError):
+        except (jose.errors.DeserializationError, TypeError):
             return AcmeResponseMalformed(message="Could not parse JWS token.")
 
         combined = self.jws.signature.combined
@@ -346,12 +352,12 @@ class AcmeBaseView(AcmeGetNonceViewMixin, View, metaclass=abc.ABCMeta):
             return AcmeResponseNotFound(message="The requested CA cannot be found.")
 
         if combined.jwk:
-            if not self.requires_key:
+            if not self.requires_key and not self.accepts_kid_or_jwk:
                 return AcmeResponseMalformed(message="Request requires a JWK key ID.")
 
             self.jwk = combined.jwk  # set JWK from request
         elif combined.kid:
-            if self.requires_key:
+            if self.requires_key and not self.accepts_kid_or_jwk:
                 return AcmeResponseMalformed(message="Request requires a full JWK key.")
 
             # combined.kid is a full URL pointing to the account.
@@ -359,6 +365,7 @@ class AcmeBaseView(AcmeGetNonceViewMixin, View, metaclass=abc.ABCMeta):
                 account = AcmeAccount.objects.viewable().get(ca=self.ca, kid=combined.kid)
             except AcmeAccount.DoesNotExist:
                 return AcmeResponseUnauthorized(message="Account not found.")
+
             if account.usable is False:
                 # RFC 855, 7.3.6:
                 #
@@ -370,7 +377,7 @@ class AcmeBaseView(AcmeGetNonceViewMixin, View, metaclass=abc.ABCMeta):
             # self.prepared['pem'] = account.pem
             # self.prepared['account_pk'] = account.pk
 
-            self.jwk = jose.JWK.load(account.pem.encode("utf-8"))  # load JWK from database
+            self.jwk = jose.jwk.JWK.load(account.pem.encode("utf-8"))  # load JWK from database
             self.account = account
         else:
             # ... 'Either "jwk" (JSON Web Key) or "kid" (Key ID)'
@@ -393,7 +400,7 @@ class AcmeBaseView(AcmeGetNonceViewMixin, View, metaclass=abc.ABCMeta):
             return AcmeResponseMalformed(message="JWS signature invalid.")
 
         # self.prepared['nonce'] = jose.encode_b64jose(combined.nonce)
-        if not self.validate_nonce(jose.encode_b64jose(combined.nonce)):
+        if combined.nonce is None or not self.validate_nonce(jose.json_util.encode_b64jose(combined.nonce)):
             # ... "nonce"
             return AcmeResponseBadNonce()
 
@@ -449,7 +456,7 @@ class AcmeMessageBaseView(AcmeBaseView, Generic[MessageTypeVar], metaclass=abc.A
         try:
             message = self.message_cls.json_loads(self.jws.payload)
             log.debug("ACME message: %s", message)
-        except jose.DeserializationError as e:
+        except jose.errors.DeserializationError as e:
             return AcmeResponseMalformedPayload(message=", ".join(e.args))
 
         return self.acme_request(message, slug)
@@ -518,7 +525,7 @@ class AcmeNewAccountView(ContactValidationMixin, AcmeMessageBaseView[messages.Re
             .decode("utf-8")
             .strip()
         )
-        thumbprint = jose.encode_b64jose(self.jwk.thumbprint())
+        thumbprint = jose.json_util.encode_b64jose(self.jwk.thumbprint())
 
         # RFC 8555, section 7.3:
         #
@@ -799,8 +806,11 @@ class AcmeOrderFinalizeView(AcmeMessageBaseView[CertificateRequest]):
 
             # We allow a client setting a CommonName, but it *must* be part of the order.
             common_name = next((attr for attr in csr.subject if attr.oid == NameOID.COMMON_NAME), None)
-            if common_name is not None and x509.DNSName(common_name.value) not in names_from_order:
-                raise AcmeBadCSR(message="CommonName was not in order.")
+            if common_name is not None:
+                if isinstance(common_name.value, bytes):  # pragma: no cover
+                    raise AcmeBadCSR(message="CommonName was not in order.")
+                if x509.DNSName(common_name.value) not in names_from_order:
+                    raise AcmeBadCSR(message="CommonName was not in order.")
 
         try:
             names_from_csr: Set[x509.Name] = set(
@@ -852,6 +862,8 @@ class AcmeOrderFinalizeView(AcmeMessageBaseView[CertificateRequest]):
                 # ready once all authorizations are valid.
                 raise AcmeForbidden(typ="orderNotReady", message="This order is not yet ready.")
 
+        # TODO: Do we validate that the account holds ALL necessary authorizations?
+
         # Parse and validate the CSR
         csr = self.validate_csr(message, authorizations)
 
@@ -1017,3 +1029,123 @@ class AcmeChallengeView(AcmePostAsGetView):
             status=challenge.status,
             validated=challenge.validated,
         )
+
+
+class AcmeCertificateRevocationView(AcmeMessageBaseView[messages.Revocation]):
+    """View providing ACMEv2 certificate revocation.
+
+    .. seealso::
+
+        `RFC 8555, section 7.6 <https://tools.ietf.org/html/rfc8555#section-7.6>`_
+    """
+
+    accepts_kid_or_jwk = True
+    message_cls = messages.Revocation
+
+    def get_certificate(self, serial: str) -> Certificate:
+        """Get the certificate that is to be revoked by this request.
+
+        This function handles the special authorization requirements for this request (they can be signed by
+        either the account key pair or the certificate key pair).
+        """
+
+        certs = Certificate.objects.filter(ca=self.ca).currently_valid()
+
+        # If the request is signed with the certificate key (and not the account), a JWK is set for this
+        # request and we verify it was signed by the certificate on record.
+        if self.jws.signature.combined.jwk:
+            # NOTE: RFC 8555, section 7.6 states:
+            #
+            #   The server MUST also consider a revocation request valid if it is signed with the private key
+            #   corresponding to the public key in the certificate.
+            #
+            # This implies that the account used to request the certificate may even be revoked or invalid, as
+            # long as the private key is used to sign the request. So we don't look at the ACME account at all
+            # here.
+            cert = certs.get(serial=serial)
+
+            jwk = cert.jwk
+
+            # The JWS signature was already verified using the JWK from the request in the base class. But we
+            # still need to verify that it was signed by the certificate that is to be revoked (and not just
+            # any certificate).
+            #
+            # Theoretically, cert.jwk != self.jwk would be a sufficient check, but we do not trust the
+            # __eq__ implementation given the linked GitHub issues, so we also verify the signature a second
+            # time based on the database key.
+            #   https://josepy.readthedocs.io/en/latest/api/util/
+            if jwk != self.jwk or not self.jws.verify(jwk):
+                raise AcmeUnauthorized(message="Request signed by the wrong certificate.")
+        else:
+            # Get the certificate by serial if it *has* an ACME account.
+            # NOTE: The base class already makes sure that the account is currently valid.
+            cert = certs.filter(acmecertificate__order__account__isnull=False).get(serial=serial)
+
+            # If the request is from the account that issued the certificate, the certificate can be revoked.
+            # NOTE: self.account is **only** set if the request has no JWK.
+            if cert.acmecertificate.order.account == self.account:
+                return cert
+
+            # If the account holds authorizations for all of the identifiers in the certificate, it can also
+            # be revoked, so get a list of all currently valid authorizations that the account holds
+            authz = set(AcmeAuthorization.objects.dns().valid().account(account=self.account).names())
+
+            # Get names from the certificate, first from the CommonName...
+            # NOTE: returns empty list if subject does not have a CommonName.
+            names = [cn.value for cn in cert.subject.get_attributes_for_oid(NameOID.COMMON_NAME)]
+
+            # ... then from the SubjectAlternativeName extension
+            try:
+                san = cert.pub.loaded.extensions.get_extension_for_class(x509.SubjectAlternativeName)
+
+                # If ANY subjectAlternativeName is NOT a DNS name, we cannot revoke this cert.
+                if list(filter(lambda v: not isinstance(v, x509.DNSName), san.value)):
+                    raise AcmeUnauthorized(message="Certificate contains non-DNS subjectAlternativeNames.")
+                names += [name.value for name in san.value]
+            except x509.ExtensionNotFound:
+                pass
+
+            # Finally test if the account holds all authorizations required for revoking this certificate.
+            if not set(names) <= authz:
+                raise AcmeUnauthorized(message="Account does not hold necessary authorizations.")
+
+        return cert
+
+    def acme_request(self, message: messages.Revocation, slug: Optional[str]) -> AcmeResponse:
+        reason_code = message.reason
+        if reason_code is None:
+            reason_code = 0
+
+        try:
+            reason = REASON_CODES[reason_code]
+        except KeyError as ex:
+            raise AcmeMalformed(
+                typ="badRevocationReason", message=f"{message.reason}: Unsupported revocation reason."
+            ) from ex
+
+        # Get cryptography certificate from ACME message
+        cg_cert = message.certificate.wrapped.to_cryptography()
+        if not isinstance(cg_cert, x509.Certificate):  # pragma: no cover
+            # COVERAGE NOTE: message deserialization already raises an error when no certificate is passed,
+            # so this check here is just for more safety (and to make mypy happy).
+            raise AcmeMalformed(message="Request did not contain a certificate.")
+
+        try:
+            cert = self.get_certificate(int_to_hex(cg_cert.serial_number))
+        except Certificate.DoesNotExist as ex:
+            raise AcmeUnauthorized(message="Certificate not found.") from ex
+
+        # Check that the certificate in the payload matches with the one on record
+        if cert.pub.loaded != cg_cert:
+            raise AcmeUnauthorized(message="Certificate does not match records.")
+
+        # RFC 8555, section 7.6
+        #
+        #   if the certificate has already been revoked, the server returns an error response with status code
+        #   400 (Bad Request) and type "urn:ietf:params:acme:error:alreadyRevoked".
+        if cert.revoked:
+            raise AcmeMalformed(typ="alreadyRevoked", message="Certificate was already revoked.")
+
+        # Finally actually revoke the certificate
+        cert.revoke(reason)
+        return AcmeResponse({})  # No response specified in RFC 8555!
