@@ -17,17 +17,15 @@
 """
 
 import copy
+import functools
 import logging
 import sys
 import typing
 from datetime import date
 from datetime import datetime
-from functools import partial
 from http import HTTPStatus
-from types import MethodType
 
 from cryptography import x509
-from cryptography.hazmat.backends import default_backend
 
 from django.contrib import admin
 from django.contrib.admin.views.main import ChangeList
@@ -49,7 +47,6 @@ from django.urls import path
 from django.urls import reverse
 from django.urls.resolvers import URLPattern
 from django.utils import timezone
-from django.utils.html import escape
 from django.utils.html import format_html
 from django.utils.translation import gettext_lazy as _
 
@@ -57,16 +54,12 @@ from django_object_actions import DjangoObjectActions
 
 from . import ca_settings
 from .constants import ReasonFlags
-from .extensions import KEY_TO_EXTENSION
 from .extensions import ExtendedKeyUsage
 from .extensions import KeyUsage
 from .extensions import SubjectAlternativeName
 from .extensions import TLSFeature
-from .extensions.base import AlternativeNameExtension
-from .extensions.base import CRLDistributionPointsBase
-from .extensions.base import NullExtension
-from .extensions.base import OrderedSetExtension
-from .extensions.base import UnrecognizedExtension
+from .extensions import get_extension_name
+from .extensions.utils import extension_as_admin_html
 from .forms import CreateCertificateForm
 from .forms import ResignCertificateForm
 from .forms import RevokeCertificateForm
@@ -83,11 +76,11 @@ from .models import X509CertMixin
 from .profiles import profiles
 from .querysets import CertificateQuerySet
 from .signals import post_issue_cert
-from .subject import Subject
 from .utils import OID_NAME_MAPPINGS
 from .utils import SERIAL_RE
 from .utils import add_colons
 from .utils import format_name
+from .utils import x509_name
 
 log = logging.getLogger(__name__)
 X509CertMixinTypeVar = typing.TypeVar("X509CertMixinTypeVar", bound=X509CertMixin)
@@ -137,20 +130,6 @@ class WatcherAdmin(WatcherAdminBase):
     """ModelAdmin for :py:class:`~django_ca.models.Watcher`."""
 
 
-if sys.version_info < (3, 7):  # pragma: py<3.7 branch
-    # In Python 3.6, Generic has a special metaclass. So does ModelAdmin in general. This creates a conflict
-    # in metaclass base classes:
-    #   https://github.com/python/typing/issues/449
-    #
-    # PYLINT NOTE: GenericMeta only exists in Python 3.6 and pylint does not get that his import is only for
-    #              that version, throwing an error for newer Python versions.
-    from typing import GenericMeta  # pylint: disable=no-name-in-module,ungrouped-imports
-
-    # PYLINT NOTE: Redefining the class with the same name is just the shortest way here
-    class MediaDefiningClass(GenericMeta, MediaDefiningClass):  # pylint: disable=function-redefined
-        """Metaclass for py3.6."""
-
-
 class CertificateMixin(
     typing.Generic[X509CertMixinTypeVar],
     MixinBase,
@@ -283,40 +262,30 @@ class CertificateMixin(
     # Properties for x509 extensions #
     ##################################
 
-    def output_template(self, obj: X509CertMixinTypeVar, key: str) -> str:
+    def output_template(self, obj: X509CertMixinTypeVar, oid: x509.ObjectIdentifier) -> str:
         """Render extension for the given object."""
 
-        ext = getattr(obj, key)
-        templates = [f"django_ca/admin/extensions/{key}.html"]
+        # PYLINT NOTE: use internal property until we can deprecate extensions
+        ext = obj._x509_extensions.get(oid)  # pylint: disable=protected-access
 
-        if isinstance(ext, NullExtension):
-            templates.append("django_ca/admin/extensions/base/null_extension.html")
-        if isinstance(ext, AlternativeNameExtension):
-            templates.append("django_ca/admin/extensions/base/alternative_name_extension.html")
-        if isinstance(ext, CRLDistributionPointsBase):
-            templates.append("django_ca/admin/extensions/base/crl_distribution_points_base.html")
-        if isinstance(ext, OrderedSetExtension):
-            templates.append("django_ca/admin/extensions/base/ordered_set_extension.html")
-        if isinstance(ext, (UnrecognizedExtension, x509.UnrecognizedExtension)):  # pragma: no cover
-            templates.append("django_ca/admin/extensions/base/unrecognized_extension.html")
-        else:
-            templates.append("django_ca/admin/extensions/base/base.html")
-        return render_to_string(templates, {"obj": obj, "extension": ext})
+        if ext is None:
+            # SubjectAlternativeName is displayed unconditionally in the main section, so a certificate
+            # without this extension will yield a KeyError in this case.
+            return render_to_string(["django_ca/admin/extensions/missing.html"])
 
-    def unknown_oid(self, oid: x509.ObjectIdentifier, obj: X509CertMixinTypeVar) -> str:
-        """Generic display for extensions that we do not know about and cannot display."""
-        ext = obj.pub.loaded.extensions.get_extension_for_oid(oid)
-        html = ""
-        if ext.critical is True:
-            text = _("Critical")
-            html = f'<img src="/static/admin/img/icon-yes.svg" alt="{text}"> {text}'
+        return extension_as_admin_html(ext)
 
-        html += f"<p>{escape(ext.value)}<p>"
-        return html
+    def __getattr__(self, name: str) -> typing.Any:
+        if name.startswith("oid_"):
+            oid = x509.ObjectIdentifier(name[4:].replace("_", "."))
+            func = functools.partial(self.output_template, oid=oid)
+            func.short_description = get_extension_name(oid)  # type: ignore[attr-defined]  # django standard
+            return func
+        raise AttributeError
 
     def get_oid_name(self, oid: x509.ObjectIdentifier) -> str:
         """Get a normalized name for the given OID."""
-        return oid.dotted_string.replace(".", "_")
+        return f"oid_{oid.dotted_string.replace('.', '_')}"
 
     # TYPE NOTE: django-stubs typehints obj as Optional[Model], but we can be more specific here
     def get_fieldsets(  # type: ignore[override] # pylint: disable=missing-function-docstring
@@ -328,32 +297,9 @@ class CertificateMixin(
             return fieldsets
 
         fieldsets = copy.deepcopy(fieldsets)
-
-        if obj.extension_fields:
-            for field in obj.extension_fields:
-                if field == SubjectAlternativeName.key:  # already displayed in main section
-                    continue
-
-                # If we encounter an object of type x509.Extension, it means that we do not yet support this
-                # extension, hence there are no accessors either. We compute a name for the extension based on
-                # the OID, create a partial function of unknown_oid and attach it under that name to this
-                # admin instance:
-                if isinstance(field, x509.Extension):
-                    oid_func = partial(self.unknown_oid, field.oid)
-                    desc = f"Unkown OID ({field.oid.dotted_string})"
-                    oid_func.short_description = desc  # type: ignore[attr-defined]
-
-                    field = self.get_oid_name(field.oid)
-
-                    # attach function to this instance
-                    setattr(self, field, oid_func)
-
-                fieldsets[self.x509_fieldset_index][1]["fields"].append(field)
-
-        else:
-            # we have no extensions, so remove the whole fieldset
-            fieldsets.pop(self.x509_fieldset_index)
-
+        for ext in obj._sorted_extensions:  # pylint: disable=protected-access
+            field = self.get_oid_name(ext.oid)
+            fieldsets[self.x509_fieldset_index][1]["fields"].append(field)
         return fieldsets
 
     def get_readonly_fields(  # type: ignore[override] # pylint: disable=missing-function-docstring
@@ -373,14 +319,9 @@ class CertificateMixin(
             # We can only change the date when the certificate was compromised if it's actually revoked.
             fields.append("compromised")
 
-        fields = list(fields)
-        for field in obj.extension_fields:
-            if isinstance(field, x509.Extension):
-                field = self.get_oid_name(field.oid)
-
-            fields.append(field)
-
-        return fields
+        # pylint: disable-next=protected-access
+        extension_fields = [self.get_oid_name(oid) for oid in obj._x509_extensions]
+        return list(fields) + extension_fields
 
     class Media:  # pylint: disable=missing-class-docstring
         css = {
@@ -388,21 +329,6 @@ class CertificateMixin(
         }
 
 
-# Attach extension properties to admin if they are not already present.
-# This makes ModelAdmin have a property for every extension that we currently support,
-# rendering as a template based on the extension key.
-for ext_key, ext_cls in KEY_TO_EXTENSION.items():
-    # Give Mixin opportunity to override method - not needed right now
-    # if hasattr(CertificateMixin, key):
-    #    continue
-
-    func = partial(CertificateMixin.output_template, key=ext_key)
-    func.short_description = ext_cls.name  # type: ignore[attr-defined]
-    bound_func = MethodType(func, CertificateMixin)
-
-    setattr(CertificateMixin, ext_key, bound_func)
-
-
 @admin.register(CertificateAuthority)
 class CertificateAuthorityAdmin(CertificateMixin[CertificateAuthority], CertificateAuthorityAdminBase):
     """ModelAdmin for :py:class:`~django_ca.models.CertificateAuthority`."""
@@ -476,9 +402,7 @@ class CertificateAuthorityAdmin(CertificateMixin[CertificateAuthority], Certific
     ]
     x509_fieldset_index = 3
 
-    # PYLINT NOTE: pylint does not recognize that function is overwritten due to generics.
-    #              https://github.com/PyCQA/pylint/issues/3605
-    def has_add_permission(self, request: HttpRequest) -> bool:  # pylint: disable=missing-function-docstring
+    def has_add_permission(self, request: HttpRequest) -> bool:
         return False
 
     # TYPE NOTE: django-stubs typehints obj as Optional[Model], but we can be more specific here
@@ -617,8 +541,8 @@ class CertificateAdmin(DjangoObjectActions, CertificateMixin[Certificate], Certi
         "distinguished_name",
         "ca",
         "hpkp_pin",
-        "subject_alternative_name",
         "profile",
+        "oid_2_5_29_17",  # SubjectAlternativeName
     ]
     search_fields = [
         "cn",
@@ -631,7 +555,7 @@ class CertificateAdmin(DjangoObjectActions, CertificateMixin[Certificate], Certi
             {
                 "fields": [
                     "cn_display",
-                    "subject_alternative_name",
+                    "oid_2_5_29_17",  # SubjectAlternativeName
                     "distinguished_name",
                     "serial_field",
                     "ca",
@@ -731,9 +655,7 @@ class CertificateAdmin(DjangoObjectActions, CertificateMixin[Certificate], Certi
     ]
     x509_fieldset_index = 1
 
-    # PYLINT NOTE: pylint does not recognize that function is overwritten due to generics.
-    #              https://github.com/PyCQA/pylint/issues/3605
-    def has_add_permission(self, request: HttpRequest) -> bool:  # pylint: disable=missing-function-docstring
+    def has_add_permission(self, request: HttpRequest) -> bool:
         # Only grant add permissions if there is at least one useable CA
         for ca in CertificateAuthority.objects.filter(enabled=True):
             if ca.key_exists:
@@ -799,15 +721,12 @@ class CertificateAdmin(DjangoObjectActions, CertificateMixin[Certificate], Certi
                 "watchers": resign_obj.watchers.all(),
             }
         else:
-            data["subject"] = Subject(
-                ca_settings.CA_PROFILES[ca_settings.CA_DEFAULT_PROFILE].get("subject", {})
-            ).name
+            profile = ca_settings.CA_PROFILES[ca_settings.CA_DEFAULT_PROFILE]
+            data["subject"] = x509_name(profile.get("subject", tuple()))
 
         return data
 
-    # PYLINT NOTE: pylint does not recognize that function is overwritten due to generics.
-    #              https://github.com/PyCQA/pylint/issues/3605
-    def add_view(  # pylint: disable=missing-function-docstring
+    def add_view(
         self,
         request: HttpRequest,
         form_url: str = "",
@@ -835,7 +754,7 @@ class CertificateAdmin(DjangoObjectActions, CertificateMixin[Certificate], Certi
             raise PermissionDenied
 
         try:
-            csr = x509.load_pem_x509_csr(request.POST["csr"].encode("ascii"), default_backend())
+            csr = x509.load_pem_x509_csr(request.POST["csr"].encode("ascii"))
         except Exception as e:  # pylint: disable=broad-except; docs don't list possible exceptions
             return JsonResponse({"message": str(e)}, status=HTTPStatus.BAD_REQUEST)
 
@@ -961,6 +880,10 @@ class CertificateAdmin(DjangoObjectActions, CertificateMixin[Certificate], Certi
         """Collapse the "Revocation" section unless the certificate is revoked."""
         fieldsets = super().get_fieldsets(request, obj=obj)
 
+        san_field_name = "oid_2_5_29_17"
+        if san_field_name in fieldsets[self.x509_fieldset_index][1]["fields"]:
+            fieldsets[self.x509_fieldset_index][1]["fields"].remove("oid_2_5_29_17")
+
         if hasattr(request, "_resign_obj"):
             return self.resign_fieldsets
         if obj is None:
@@ -997,9 +920,7 @@ class CertificateAdmin(DjangoObjectActions, CertificateMixin[Certificate], Certi
     expires_date.admin_order_field = "expires"  # type: ignore[attr-defined] # django standard
 
     # TYPE NOTE: django-stubs typehints obj as Model, but we can be more specific here
-    # PYLINT NOTE: pylint does not recognize that function is overwritten due to generics.
-    #              https://github.com/PyCQA/pylint/issues/3605
-    def save_model(  # type: ignore[override] # pylint: disable=missing-function-docstring
+    def save_model(  # type: ignore[override]
         self,
         request: HttpRequest,
         obj: Certificate,
