@@ -16,6 +16,7 @@
 import os
 import re
 import typing
+import warnings
 from datetime import timedelta
 
 from cryptography.hazmat.primitives import hashes
@@ -27,6 +28,8 @@ from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
 from django.utils.translation import gettext_lazy as _
 
+from .deprecation import RemovedInDjangoCA123Warning
+
 if "CA_DIR" in os.environ:  # pragma: no cover
     CA_DIR = os.path.join(os.environ["CA_DIR"], "files")
 else:
@@ -169,15 +172,29 @@ if CA_DEFAULT_CA != "0":
 if re.search("[^0-9A-F]", CA_DEFAULT_CA):
     raise ImproperlyConfigured(f"CA_DEFAULT_CA: {CA_DEFAULT_CA}: Serial contains invalid characters.")
 
-CA_DEFAULT_SUBJECT = getattr(settings, "CA_DEFAULT_SUBJECT", {})
+_SUBJECT_AS_DICT_MAPPING = "%s as a dict wil be removed in django-ca==1.23. Please use a tuple instead."
+CA_DEFAULT_SUBJECT: typing.Tuple[typing.Tuple[str, str], ...] = getattr(
+    settings, "CA_DEFAULT_SUBJECT", tuple()
+)
+if isinstance(CA_DEFAULT_SUBJECT, dict):
+    # pylint: disable=consider-using-f-string # don't want to use f-string here
+    warnings.warn(_SUBJECT_AS_DICT_MAPPING % "CA_DEFAULT_SUBJECT", category=RemovedInDjangoCA123Warning)
+    CA_DEFAULT_SUBJECT = tuple(CA_DEFAULT_SUBJECT.items())
 
 # Add ability just override/add some profiles
 _CA_PROFILE_OVERRIDES = getattr(settings, "CA_PROFILES", {})
 for name, profile in _CA_PROFILE_OVERRIDES.items():
     if profile is None:
         del CA_PROFILES[name]
+        continue
 
-    elif name in CA_PROFILES:
+    if isinstance(profile.get("subject"), dict):
+        warnings.warn(
+            _SUBJECT_AS_DICT_MAPPING % f"{name}: Profile subject", category=RemovedInDjangoCA123Warning
+        )
+        profile["subject"] = tuple(profile["subject"].items())
+
+    if name in CA_PROFILES:
         CA_PROFILES[name].update(profile)
     else:
         CA_PROFILES[name] = profile
@@ -204,6 +221,7 @@ CA_CRL_PROFILES: typing.Dict[str, typing.Dict[str, typing.Any]] = getattr(
 CA_PASSWORDS: typing.Dict[str, str] = getattr(settings, "CA_PASSWORDS", {})
 
 # ACME settings
+CA_ENABLE_ACME = getattr(settings, "CA_ENABLE_ACME", False)
 ACME_ORDER_VALIDITY: timedelta = getattr(settings, "CA_ACME_ORDER_VALIDITY", timedelta(hours=1))
 ACME_ACCOUNT_REQUIRES_CONTACT = getattr(settings, "CA_ACME_ACCOUNT_REQUIRES_CONTACT", True)
 ACME_MAX_CERT_VALIDITY = getattr(settings, "CA_ACME_MAX_CERT_VALIDITY", timedelta(days=90))
@@ -260,8 +278,6 @@ CA_FILE_STORAGE_KWARGS = getattr(
 
 CA_FILE_STORAGE_URL = "https://django-ca.readthedocs.io/en/latest/update.html#update-to-1-12-0-or-later"
 
-CA_ENABLE_ACME = getattr(settings, "CA_ENABLE_ACME", False)
-
 # Decide if we should use Celery or not
 CA_USE_CELERY = getattr(settings, "CA_USE_CELERY", None)
 if CA_USE_CELERY is None:
