@@ -18,8 +18,11 @@ import argparse
 import io
 import sys
 import typing
+from datetime import datetime
 from datetime import timedelta
 
+from cryptography import x509
+
 from django.core.management.base import BaseCommand as _BaseCommand
 from django.core.management.base import CommandError
 from django.core.management.base import CommandParser
@@ -29,10 +32,10 @@ from django.utils import timezone
 from .. import ca_settings
 from ..extensions import ExtendedKeyUsage
 from ..extensions import KeyUsage
-from ..extensions import SubjectAlternativeName
 from ..extensions import TLSFeature
 from ..models import CertificateAuthority
-from ..utils import SUBJECT_FIELDS
+from ..profiles import Profile
+from ..utils import NAME_OID_MAPPINGS
 from . import actions
 from . import mixins
 
@@ -113,7 +116,7 @@ class BaseCommand(mixins.ArgumentsMixin, _BaseCommand, metaclass=abc.ABCMeta):
     @property
     def valid_subject_keys(self) -> str:
         """Return human-readable enumeration of valid subject keys (CN/...)."""
-        fields = [f'"{f}"' for f in SUBJECT_FIELDS]
+        fields = [f'"{f}"' for f in NAME_OID_MAPPINGS]
         return f"{', '.join(fields[:-1])} and {fields[-1]}"
 
     def add_subject(
@@ -128,13 +131,13 @@ class BaseCommand(mixins.ArgumentsMixin, _BaseCommand, metaclass=abc.ABCMeta):
 
     def add_ecc_curve(self, parser: CommandParser) -> None:
         """Add --ecc-curve option."""
-        default = ca_settings.CA_DEFAULT_ECC_CURVE.__class__.__name__
+        default = ca_settings.CA_DEFAULT_ECC_CURVE()
         parser.add_argument(
             "--ecc-curve",
             metavar="CURVE",
             action=actions.KeyCurveAction,
-            default=ca_settings.CA_DEFAULT_ECC_CURVE(),
-            help=f"Elliptic Curve used for ECC keys (default: {default}).",
+            default=default,
+            help=f"Elliptic Curve used for ECC keys (default: {default.name}).",
         )
 
     def add_key_size(self, parser: CommandParser) -> None:
@@ -173,21 +176,12 @@ class BaseSignCommand(BaseCommand):  # pylint: disable=abstract-method; is a bas
     """Base class for commands signing certificates (sign_cert, resign_cert)."""
 
     add_extensions_help = ""  # concrete classes should set this
-    sign_extensions: typing.Set[
-        typing.Type[
-            typing.Union[
-                TLSFeature,
-                SubjectAlternativeName,
-                KeyUsage,
-                ExtendedKeyUsage,
-            ]
-        ]
-    ] = {
-        SubjectAlternativeName,
+    sign_extensions: typing.Set[typing.Type[typing.Union[TLSFeature, KeyUsage, ExtendedKeyUsage]]] = {
         KeyUsage,
         ExtendedKeyUsage,
         TLSFeature,
     }
+    cg_sign_extensions: typing.Tuple[typing.Type[x509.ExtensionType], ...] = (x509.SubjectAlternativeName,)
     subject_help = ""  # concrete classes should set this
 
     def add_base_args(self, parser: CommandParser, no_default_ca: bool = False) -> None:
@@ -200,15 +194,14 @@ class BaseSignCommand(BaseCommand):  # pylint: disable=abstract-method; is a bas
 
         parser.add_argument(
             "--expires",
-            default=ca_settings.CA_DEFAULT_EXPIRES,
             action=actions.ExpiresAction,
-            help="Sign the certificate for DAYS days (default: %(default)s)",
+            help=f"Sign the certificate for DAYS days (default: {ca_settings.CA_DEFAULT_EXPIRES})",
         )
         parser.add_argument(
             "--alt",
             metavar="DOMAIN",
             action=actions.AlternativeNameAction,
-            extension=SubjectAlternativeName,
+            extension_type=x509.SubjectAlternativeName,
             help="Add a subjectAltName to the certificate (may be given multiple times)",
         )
         parser.add_argument(
@@ -264,13 +257,16 @@ class BaseSignCommand(BaseCommand):  # pylint: disable=abstract-method; is a bas
     def test_options(  # pylint: disable=unused-argument
         self,
         ca: CertificateAuthority,
-        expires: timedelta,
+        expires: typing.Optional[typing.Union[datetime, timedelta]],
         password: typing.Optional[bytes],
+        profile: Profile,
         **options: typing.Any,
     ) -> None:
         """Additional tests for validity of some options."""
 
-        if ca.expires < timezone.now() + expires:
+        expires = profile.get_expires(expires)
+
+        if ca.expires < expires:
             max_days = (ca.expires - timezone.now()).days
             raise CommandError(
                 f"Certificate would outlive CA, maximum expiry for this CA is {max_days} days."
