@@ -21,15 +21,15 @@ import typing
 from datetime import timedelta
 
 from cryptography import x509
-from cryptography.hazmat.backends import default_backend
 
 from django.core.management.base import CommandError
 from django.core.management.base import CommandParser
 from django.utils import timezone
 
 from ... import ca_settings
+from ...extensions import OID_DEFAULT_CRITICAL
+from ...extensions import OID_TO_KEY
 from ...extensions import Extension
-from ...extensions import SubjectAlternativeName
 from ...management.base import BaseSignCommand
 from ...models import Certificate
 from ...models import CertificateAuthority
@@ -106,7 +106,7 @@ https://django-ca.readthedocs.io/en/latest/extensions.html for more information.
         self,
         ca: CertificateAuthority,
         subject: typing.Optional[Subject],
-        expires: timedelta,
+        expires: typing.Optional[timedelta],
         watch: typing.List[str],
         password: typing.Optional[bytes],
         cn_in_san: bool,
@@ -120,20 +120,33 @@ https://django-ca.readthedocs.io/en/latest/extensions.html for more information.
             raise CommandError("Certificate Authority has expired.")
         if ca.revoked:
             raise CommandError("Certificate Authority is revoked.")
-        self.test_options(ca=ca, expires=expires, password=password, **options)
+        profile_obj = profiles[profile]
+        self.test_options(ca=ca, expires=expires, password=password, profile=profile_obj, **options)
         subject = subject or Subject()
 
         # get list of watchers
         watchers = [Watcher.from_addr(addr) for addr in watch]
 
         # get extensions based on profiles
-        extensions: typing.List[Extension[x509.ExtensionType, typing.Any, typing.Any]] = []
+        extensions: typing.Dict[
+            str,
+            typing.Union[
+                Extension[x509.ExtensionType, typing.Any, typing.Any], x509.Extension[x509.ExtensionType]
+            ],
+        ] = {}
 
         for ext in self.sign_extensions:
             if options[ext.key]:
-                extensions.append(options[ext.key])
+                extensions[ext.key] = options[ext.key]
+        for cg_ext in self.cg_sign_extensions:
+            ext_key = OID_TO_KEY[cg_ext.oid]
+            if options[ext_key]:
+                ext_type = options[ext_key]
+                extensions[ext_key] = x509.Extension(
+                    critical=OID_DEFAULT_CRITICAL[cg_ext.oid], value=ext_type, oid=cg_ext.oid
+                )
 
-        if "CN" not in subject and not options[SubjectAlternativeName.key]:
+        if "CN" not in subject and not options[OID_TO_KEY[x509.SubjectAlternativeName.oid]]:
             raise CommandError("Must give at least a CN in --subject or one or more --alt arguments.")
 
         # Read the CSR
@@ -150,17 +163,16 @@ https://django-ca.readthedocs.io/en/latest/extensions.html for more information.
                 csr_bytes = csr_stream.read()
 
         if csr_bytes.startswith(b"-----BEGIN CERTIFICATE REQUEST-----"):
-            csr = x509.load_pem_x509_csr(csr_bytes, default_backend())
+            csr = x509.load_pem_x509_csr(csr_bytes)
         else:
-            csr = x509.load_der_x509_csr(csr_bytes, default_backend())
+            csr = x509.load_der_x509_csr(csr_bytes)
 
         try:
             cert = Certificate.objects.create_cert(
                 ca,
                 csr,
-                profile=profiles[profile],
+                profile=profile_obj,
                 cn_in_san=cn_in_san,
-                # TODO: since expires option has a default, it currently overrides profile values
                 expires=expires,
                 extensions=extensions,
                 password=password,
