@@ -29,9 +29,9 @@ from typing import TypeVar
 from typing import Union
 
 from cryptography import x509
-from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import serialization
 from cryptography.hazmat.primitives.asymmetric import ec
+from cryptography.hazmat.primitives.hashes import HashAlgorithm
 from cryptography.hazmat.primitives.serialization import Encoding
 from cryptography.hazmat.primitives.serialization import PrivateFormat
 from cryptography.x509.oid import AuthorityInformationAccessOID
@@ -42,6 +42,10 @@ from django.urls import reverse
 
 from . import ca_settings
 from .deprecation import RemovedInDjangoCA122Warning
+from .deprecation import RemovedInDjangoCA123Warning
+from .deprecation import deprecate_argument
+from .deprecation import deprecate_type
+from .extensions import OID_DEFAULT_CRITICAL
 from .extensions import Extension
 from .extensions import IssuerAlternativeName
 from .extensions import NameConstraints
@@ -56,13 +60,14 @@ from .signals import pre_create_ca
 from .subject import Subject
 from .typehints import Expires
 from .typehints import ParsableExtension
-from .typehints import ParsableHash
 from .typehints import ParsableKeyType
 from .utils import ca_storage
+from .utils import format_general_name
 from .utils import generate_private_key
 from .utils import get_cert_builder
 from .utils import int_to_hex
 from .utils import parse_expires
+from .utils import parse_general_name
 from .utils import parse_hash_algorithm
 from .utils import sort_name
 from .utils import validate_hostname
@@ -78,6 +83,7 @@ if TYPE_CHECKING:
     from .models import AcmeOrder
     from .models import Certificate
     from .models import CertificateAuthority
+    from .querysets import AcmeAccountQuerySet
     from .querysets import CertificateAuthorityQuerySet
     from .querysets import CertificateQuerySet
 
@@ -172,12 +178,14 @@ class CertificateManagerMixin(Generic[X509CertMixinTypeVar, QuerySetTypeVar]):
     def _extra_extensions(
         self,
         builder: x509.CertificateBuilder,
-        extra_extensions: Iterable[Union["x509.Extension[x509.ExtensionType]", "Extension[Any, Any, Any]"]],
+        extra_extensions: typing.List[typing.Union[x509.Extension[x509.ExtensionType]]],
     ) -> x509.CertificateBuilder:
+        warn = "Passing a django_ca.extensions.Extension is deprecated and will be removed in django_ca 1.23."
         for ext in extra_extensions:
             if isinstance(ext, x509.Extension):
                 builder = builder.add_extension(ext.value, critical=ext.critical)
             elif isinstance(ext, Extension):
+                warnings.warn(warn, category=RemovedInDjangoCA123Warning, stacklevel=2)
                 builder = builder.add_extension(*ext.for_builder())
             else:
                 raise ValueError(f"Cannot add extension of type {type(ext).__name__}")
@@ -213,31 +221,33 @@ class CertificateAuthorityManager(
         def usable(self) -> "CertificateAuthorityQuerySet":
             ...
 
+    @deprecate_argument("name_constraints", RemovedInDjangoCA123Warning)
+    @deprecate_type("issuer_alt_name", (str, IssuerAlternativeName), RemovedInDjangoCA123Warning)
     def init(
         self,
         name: str,
         subject: x509.Name,
         expires: Expires = None,
-        algorithm: ParsableHash = None,
+        algorithm: typing.Optional[HashAlgorithm] = None,
         parent: Optional["CertificateAuthority"] = None,
         default_hostname: Optional[Union[bool, str]] = None,
         pathlen: Optional[int] = None,
         issuer_url: Optional[str] = None,
-        issuer_alt_name: Union[str, IssuerAlternativeName] = "",
+        issuer_alt_name: typing.Optional[x509.Extension[x509.IssuerAlternativeName]] = None,
         crl_url: Optional[Iterable[str]] = None,
         ocsp_url: Optional[str] = None,
         ca_issuer_url: Optional[str] = None,
         ca_crl_url: Optional[Sequence[str]] = None,
         ca_ocsp_url: Optional[str] = None,
         name_constraints: Optional[Union[ParsableExtension, NameConstraints]] = None,
-        password: Optional[bytes] = None,
+        permitted_subtrees: typing.Optional[typing.Iterable[x509.GeneralName]] = None,
+        excluded_subtrees: typing.Optional[typing.Iterable[x509.GeneralName]] = None,
+        password: Optional[Union[str, bytes]] = None,
         parent_password: Optional[Union[str, bytes]] = None,
         ecc_curve: Optional[ec.EllipticCurve] = None,
         key_type: ParsableKeyType = "RSA",
         key_size: Optional[int] = None,
-        extra_extensions: Optional[
-            typing.Collection[Union["x509.Extension[x509.ExtensionType]", "Extension[Any, Any, Any]"]]
-        ] = None,
+        extra_extensions: Optional[typing.Iterable[x509.Extension[x509.ExtensionType]]] = None,
         path: Union[pathlib.PurePath, str] = "ca",
         caa: str = "",
         website: str = "",
@@ -255,6 +265,17 @@ class CertificateAuthorityManager(
 
            * Passing a ``str`` or :py:class:`~django_ca.subject.Subject` for ``subject``.
 
+        .. deprecated:: 1.21.0
+
+           * The `name_constraints` parameter is deprecated and will be removed in ``django_ca==1.23``. Use
+             the `permitted_subtrees` and `excluded_subtrees` parameter instead.
+           * Passing  ``django_ca.extensions.Extension`` instance to `extra_extensions` is now deprecated. The
+             feature will be removed in ``django_ca==1.23``. Pass a ``x509.Extension`` instance instead.
+           * The `issuer_alt_name` now accepts a
+             :py:class:`~cg:cryptography.x509.Extension` with a
+             :py:class:`~cg:cryptography.x509.IssuerAlternativeName` value, passing a `str` or
+             ``django_ca.extensions.IssuerAlternativeName`` is deprecated and will be removed in
+             ``django_ca==1.23``.
 
         Parameters
         ----------
@@ -266,10 +287,9 @@ class CertificateAuthorityManager(
         expires : int or datetime or timedelta, optional
             When this certificate authority will expire, defaults to :ref:`CA_DEFAULT_EXPIRES
             <settings-ca-default-expires>`.
-        algorithm : str or :py:class:`~cg:cryptography.hazmat.primitives.hashes.HashAlgorithm`, optional
-            Hash algorithm used when signing the certificate, passed to
-            :py:func:`~django_ca.utils.parse_hash_algorithm`. The default is the value of the
-            :ref:`CA_DIGEST_ALGORITHM <settings-ca-digest-algorithm>` setting.
+        algorithm : :py:class:`~cg:cryptography.hazmat.primitives.hashes.HashAlgorithm`, optional
+            Hash algorithm used when signing the certificate, defaults to
+            :ref:`CA_DIGEST_ALGORITHM <settings-ca-digest-algorithm>`.
         parent : :py:class:`~django_ca.models.CertificateAuthority`, optional
             Parent certificate authority for the new CA. Passing this value makes the CA an intermediate
             authority. Let unset if this CA will be used for OpenSSH.
@@ -281,10 +301,9 @@ class CertificateAuthorityManager(
             extension.
         issuer_url : str
             URL for the DER/ASN1 formatted certificate that is signing certificates.
-        issuer_alt_name : :py:class:`~django_ca.extensions.IssuerAlternativeName` or str, optional
-            IssuerAlternativeName used when signing certificates. If the value is not an instance of
-            :py:class:`~django_ca.extensions.IssuerAlternativeName`, it will be passed as argument to
-            the constructor of the class.
+        issuer_alt_name : :py:class:`~cg:cryptography.x509.Extension`, optional
+            IssuerAlternativeName used when signing certificates.  The value of the extension must be an
+            :py:class:`~cg:cryptography.x509.IssuerAlternativeName` instance.
         crl_url : list of str, optional
             CRL URLs used for certificates signed by this CA.
         ocsp_url : str, optional
@@ -298,13 +317,15 @@ class CertificateAuthorityManager(
         ca_ocsp_url : str, optional
             OCSP URL used for this CA. This value is only meaningful for intermediate CAs. The default is
             no value, unless :ref:`CA_DEFAULT_HOSTNAME <settings-ca-default-hostname>` is set.
-        name_constraints : list of lists or :py:class:`~django_ca.extensions.NameConstraints`
-            List of names that this CA can sign and/or cannot sign. The value is passed to
-            :py:class:`~django_ca.extensions.NameConstraints` if the value is not already an instance of that
-            class.
-        password : bytes, optional
+        permitted_subtrees : list of x509.GeneralName, optional
+            List of general names to add to the permitted names of the NameConstraints extension.
+        excluded_subtrees : list of x509.GeneralName, optional
+            List of general names to add to the permitted names of the NameConstraints extension.
+        name_constraints : :py:class:`~django_ca.extensions.NameConstraints`
+            Deprecated in favor of `permitted_subtrees` and `excluded_subtrees`.
+        password : bytes or str, optional
             Password to encrypt the private key with.
-        parent_password : bytes, optional
+        parent_password : bytes or str, optional
             Password that the private key of the parent CA is encrypted with.
         ecc_curve : :py:class:`~cg:cryptography.hazmat.primitives.asymmetric.ec.EllipticCurve`, optional
             An elliptic curve to use for ECC keys. This parameter is ignored if ``key_type`` is not ``"ECC"``.
@@ -316,8 +337,7 @@ class CertificateAuthorityManager(
             Integer specifying the key size, must be a power of two (e.g. 2048, 4096, ...). Defaults to
             the :ref:`CA_DEFAULT_KEY_SIZE <settings-ca-default-key-size>`, unused if
             ``key_type="ECC"`` or ``key_type="EdDSA"``.
-        extra_extensions : list of :py:class:`cg:cryptography.x509.Extension` or \
-                :py:class:`django_ca.extensions.base.Extension`, optional
+        extra_extensions : list of :py:class:`cg:cryptography.x509.Extension`
             An optional list of additional extensions to add to the certificate.
         path : str or pathlib.PurePath, optional
             Where to store the CA private key (default ``ca``).
@@ -347,10 +367,17 @@ class CertificateAuthorityManager(
         #       directly. generate_private_key() invokes this again, but we here to avoid sending a signal.
         validate_key_parameters(key_size, key_type, ecc_curve)
 
-        if not openssh_ca:
-            algorithm = parse_hash_algorithm(algorithm)
-        else:
+        if openssh_ca:
             algorithm = None
+        elif algorithm is None:
+            algorithm = ca_settings.CA_DIGEST_ALGORITHM
+        elif not isinstance(algorithm, HashAlgorithm):
+            warnings.warn(
+                f"Passing a {algorithm.__class__.__name__} as algorithm is deprecated",
+                category=RemovedInDjangoCA122Warning,
+                stacklevel=1,
+            )
+            algorithm = parse_hash_algorithm(algorithm)
         expires = parse_expires(expires)
 
         if openssh_ca and parent:
@@ -366,29 +393,27 @@ class CertificateAuthorityManager(
 
         # Append OpenSSH extensions if an OpenSSH CA was requested
         if openssh_ca:
-            # TYPE NOTE: This seems to be a false positive
-            extra_extensions.extend([SshHostCaExtension(), SshUserCaExtension()])  # type: ignore[list-item]
+            extra_extensions.extend([SshHostCaExtension(), SshUserCaExtension()])
 
         # Handle old types for subject
         _subject_warning = f"Passing a {subject.__class__.__name__} as subject is deprecated"
         if isinstance(subject, Subject):
-            warnings.warn(
-                _subject_warning,
-                category=RemovedInDjangoCA122Warning,
-                stacklevel=1,
-            )
+            warnings.warn(_subject_warning, category=RemovedInDjangoCA122Warning, stacklevel=1)
             subject = subject.name
         elif not isinstance(subject, x509.Name):
-            warnings.warn(
-                _subject_warning,
-                category=RemovedInDjangoCA122Warning,
-                stacklevel=1,
-            )
+            warnings.warn(_subject_warning, category=RemovedInDjangoCA122Warning, stacklevel=1)
             subject = sort_name(x509_name(subject))
 
         # Normalize extensions to django_ca.extensions.Extension subclasses
-        if issuer_alt_name and not isinstance(issuer_alt_name, IssuerAlternativeName):
-            issuer_alt_name = IssuerAlternativeName({"value": [issuer_alt_name]})
+        if isinstance(issuer_alt_name, str):
+            issuer_alt_name = x509.Extension(
+                oid=x509.IssuerAlternativeName.oid,
+                critical=OID_DEFAULT_CRITICAL[x509.IssuerAlternativeName.oid],
+                value=x509.IssuerAlternativeName(general_names=[parse_general_name(issuer_alt_name)]),
+            )
+        elif isinstance(issuer_alt_name, IssuerAlternativeName):
+            issuer_alt_name = issuer_alt_name.as_extension()
+
         if crl_url is None:
             crl_url = []
 
@@ -447,6 +472,8 @@ class CertificateAuthorityManager(
             ca_crl_url=ca_crl_url,
             ca_ocsp_url=ca_ocsp_url,
             name_constraints=name_constraints,
+            permitted_subtrees=permitted_subtrees,
+            excluded_subtrees=excluded_subtrees,
             password=password,
             parent_password=parent_password,
             extra_extensions=extra_extensions,
@@ -495,7 +522,11 @@ class CertificateAuthorityManager(
         for critical, ext in self.get_common_extensions(ca_issuer_url, ca_crl_url, ca_ocsp_url):
             builder = builder.add_extension(ext, critical=critical)
 
-        if name_constraints:
+        if permitted_subtrees is not None or excluded_subtrees is not None:
+            builder = builder.add_extension(
+                x509.NameConstraints(permitted_subtrees, excluded_subtrees), critical=True
+            )
+        elif name_constraints:
             if not isinstance(name_constraints, NameConstraints):
                 name_constraints = NameConstraints(name_constraints)
 
@@ -504,17 +535,20 @@ class CertificateAuthorityManager(
         if extra_extensions:
             builder = self._extra_extensions(builder, extra_extensions)
 
-        certificate = builder.sign(
-            private_key=private_sign_key, algorithm=algorithm, backend=default_backend()
-        )
+        certificate = builder.sign(private_key=private_sign_key, algorithm=algorithm)
 
         # Normalize extensions for create()
         crl_url = "\n".join(crl_url)
 
+        # Convert arguments for database storage
+        serialized_ian = ""
+        if issuer_alt_name is not None:
+            serialized_ian = ",".join(format_general_name(name) for name in issuer_alt_name.value)
+
         ca = self.model(
             name=name,
             issuer_url=issuer_url,
-            issuer_alt_name=",".join(issuer_alt_name),
+            issuer_alt_name=serialized_ian,
             ocsp_url=ocsp_url,
             crl_url=crl_url,
             parent=parent,
@@ -529,6 +563,8 @@ class CertificateAuthorityManager(
         if password is None:
             encryption: serialization.KeySerializationEncryption = serialization.NoEncryption()
         else:
+            if isinstance(password, str):
+                password = password.encode("utf-8")
             encryption = serialization.BestAvailableEncryption(password)
 
         pem = private_key.private_bytes(
@@ -617,6 +653,14 @@ class CertificateManager(
 class AcmeAccountManager(AcmeAccountManagerBase):
     """Model manager for :py:class:`~django_ca.models.AcmeAccount`."""
 
+    if TYPE_CHECKING:
+        # See CertificateManagerMixin for description on this branch
+        #
+        # pylint: disable=missing-function-docstring,unused-argument; just defining stubs here
+
+        def viewable(self) -> "AcmeAccountQuerySet":
+            ...
+
 
 class AcmeOrderManager(AcmeOrderManagerBase):
     """Model manager for :py:class:`~django_ca.models.AcmeOrder`."""
