@@ -36,10 +36,14 @@ from typing import Tuple
 from typing import Union
 from typing import cast
 
+import josepy as jose
+from acme import challenges
+from acme import messages
+
 from cryptography import x509
-from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import hashes
 from cryptography.hazmat.primitives import serialization
+from cryptography.hazmat.primitives.asymmetric import dh
 from cryptography.hazmat.primitives.asymmetric import dsa
 from cryptography.hazmat.primitives.asymmetric import ec
 from cryptography.hazmat.primitives.asymmetric import ed448
@@ -49,6 +53,8 @@ from cryptography.hazmat.primitives.serialization import Encoding
 from cryptography.hazmat.primitives.serialization import PrivateFormat
 from cryptography.hazmat.primitives.serialization import PublicFormat
 from cryptography.hazmat.primitives.serialization import load_pem_private_key
+from cryptography.x509.oid import AuthorityInformationAccessOID
+from cryptography.x509.oid import ExtensionOID
 from cryptography.x509.oid import NameOID
 
 from django.conf import settings
@@ -114,13 +120,13 @@ from .querysets import CertificateAuthorityQuerySet
 from .querysets import CertificateQuerySet
 from .signals import post_revoke_cert
 from .signals import pre_revoke_cert
-from .typehints import PRIVATE_KEY_TYPES
 from .typehints import Expires
 from .typehints import ExtensionTypeTypeVar
 from .typehints import Literal
 from .typehints import ParsableHash
 from .typehints import ParsableKeyType
 from .typehints import ParsableValue
+from .typehints import PrivateKeyTypes
 from .typehints import SerializedValue
 from .utils import bytes_to_hex
 from .utils import ca_storage
@@ -132,6 +138,7 @@ from .utils import int_to_hex
 from .utils import multiline_url_validator
 from .utils import parse_encoding
 from .utils import parse_expires
+from .utils import parse_general_name
 from .utils import parse_hash_algorithm
 from .utils import parse_key_curve
 from .utils import read_file
@@ -139,13 +146,12 @@ from .utils import validate_key_parameters
 
 log = logging.getLogger(__name__)
 
-try:
-    # Optional ACME imports
-    import josepy as jose
-    from acme import challenges
-    from acme import messages
-except ImportError:  # pragma: nocover
-    jose = challenges = messages = None  # type: ignore[assignment]
+_UNSUPPORTED_PRIVATE_KEY_TYPES = (
+    dh.DHPrivateKey,
+    ed448.Ed448PrivateKey,
+    x25519.X25519PrivateKey,
+    x448.X448PrivateKey,
+)
 
 
 def acme_slug() -> str:
@@ -348,7 +354,7 @@ class X509CertMixin(DjangoCAModel):
         """
         self.pub = LazyCertificate(value)
         self.cn = next(  # pylint: disable=invalid-name
-            (attr.value for attr in value.subject if attr.oid == NameOID.COMMON_NAME), ""
+            (attr.value for attr in value.subject if attr.oid == NameOID.COMMON_NAME), ""  # type: ignore
         )
         self.expires = self.not_after
         self.valid_from = self.not_before
@@ -368,9 +374,22 @@ class X509CertMixin(DjangoCAModel):
         return self.pub.loaded.signature_hash_algorithm
 
     def get_digest(self, algo: ParsableHash) -> str:
+        """Get the fingerprint for this certificate.
+
+        .. deprecated:: 1.21.0
+
+           Use :py:func:`~django_ca.models.X509CertMixin.get_fingerprint` instead.
+        """
+        warnings.warn(
+            "get_digest() is deprecated, use get_fingerprint() instead",
+            category=RemovedInDjangoCA122Warning,
+            stacklevel=1,
+        )
+        return self.get_fingerprint(parse_hash_algorithm(algo))
+
+    def get_fingerprint(self, algorithm: hashes.HashAlgorithm) -> str:
         """Get the digest for a certificate as string, including colons."""
-        algo = parse_hash_algorithm(algo)
-        return bytes_to_hex(self.pub.loaded.fingerprint(algo))
+        return bytes_to_hex(self.pub.loaded.fingerprint(algorithm))
 
     def get_filename(self, ext: str, bundle: bool = False) -> str:
         """Get a filename safe for any file system and OS for this certificate based on the common name.
@@ -430,7 +449,7 @@ class X509CertMixin(DjangoCAModel):
             # RFC 5280, 5.3.2 says that this extension MUST be non-critical
             revoked_cert = revoked_cert.add_extension(x509.InvalidityDate(compromised), critical=False)
 
-        return revoked_cert.build(default_backend())
+        return revoked_cert.build()
 
     @property
     def hpkp_pin(self) -> str:
@@ -457,6 +476,19 @@ class X509CertMixin(DjangoCAModel):
         """
         return self.pub.loaded.issuer
 
+    @property
+    def jwk(self) -> typing.Union[jose.jwk.JWKRSA, jose.jwk.JWKEC]:
+        """Get a JOSE JWK public key for this certificate."""
+
+        pkey = self.pub.loaded.public_key()
+        jwk = jose.jwk.JWK.load(pkey.public_bytes(Encoding.DER, PublicFormat.SubjectPublicKeyInfo))
+
+        # JWK.load() may return a private instead, so we rule this out here for type safety. This branch
+        # should normally not happen.
+        if not isinstance(jwk, (jose.jwk.JWKRSA, jose.jwk.JWKEC)):  # pragma: no cover
+            raise TypeError(f"Loading JWK RSA key returned {type(jwk)}.")
+        return jwk
+
     @property
     def not_before(self) -> datetime:
         """Date/Time this certificate was created"""
@@ -519,26 +551,7 @@ class X509CertMixin(DjangoCAModel):
     def _sorted_extensions(self) -> List["x509.Extension[x509.ExtensionType]"]:
         # NOTE: We need the dotted_string in the sort key if we have multiple unknown extensions, which then
         #       show up as "Unknown OID" and have to be sorted by oid
-        return list(
-            sorted(self._x509_extensions.values(), key=lambda e: (get_extension_name(e), e.oid.dotted_string))
-        )
-
-    @cached_property
-    def extension_fields(self) -> List[Union["x509.Extension[x509.ExtensionType]", str]]:
-        """List of all extensions fields for this certificate."""
-        fields: List[Union["x509.Extension[x509.ExtensionType]", str]] = []
-
-        for ext in self._sorted_extensions:
-            if ext.oid in OID_TO_EXTENSION:
-                fields.append(OID_TO_EXTENSION[ext.oid].key)
-
-            # extension that does not support new extension framework
-            else:
-                log.warning(
-                    "Unknown extension encountered: %s (%s)", get_extension_name(ext), ext.oid.dotted_string
-                )
-                fields.append(ext)
-        return fields
+        return list(sorted(self._x509_extensions.values(), key=lambda e: get_extension_name(e.oid)))
 
     @cached_property
     def extensions(
@@ -810,7 +823,7 @@ class CertificateAuthority(X509CertMixin):
 
     _key = None
 
-    def key(self, password: Optional[Union[str, bytes]] = None) -> PRIVATE_KEY_TYPES:
+    def key(self, password: Optional[Union[str, bytes]] = None) -> PrivateKeyTypes:
         """The CAs private key as private key.
 
         .. seealso:: :py:func:`~cg:cryptography.hazmat.primitives.serialization.load_pem_private_key`.
@@ -822,12 +835,12 @@ class CertificateAuthority(X509CertMixin):
             key_data = read_file(self.private_key_path)
 
             try:
-                self._key = load_pem_private_key(key_data, password, default_backend())
+                self._key = load_pem_private_key(key_data, password)
             except ValueError as ex:
                 # cryptography passes the OpenSSL error directly here and it is notoriously unstable.
                 raise ValueError("Could not decrypt private key - bad password?") from ex
-        if isinstance(self._key, ed448.Ed448PrivateKey):  # pragma: nocover
-            raise ValueError("Ed448 private keys are not supported.")
+        if isinstance(self._key, _UNSUPPORTED_PRIVATE_KEY_TYPES):  # pragma: no cover
+            raise ValueError("Private key of this type is not supported.")
 
         return self._key
 
@@ -972,7 +985,7 @@ class CertificateAuthority(X509CertMixin):
         csr = (
             x509.CertificateSigningRequestBuilder()
             .subject_name(self.pub.loaded.subject)
-            .sign(private_key, hashes.SHA256(), default_backend())
+            .sign(private_key, hashes.SHA256())
         )
 
         # TODO: The subject we pass is just a guess - see what public CAs do!?  pylint: disable=fixme
@@ -998,43 +1011,53 @@ class CertificateAuthority(X509CertMixin):
                 ca_storage.save(path, ContentFile(contents))
         return private_path, cert_path, cert
 
+    def get_authority_information_access_extension(
+        self,
+    ) -> typing.Optional[x509.Extension[x509.AuthorityInformationAccess]]:
+        """Get the AuthorityInformationAccess extension to use in certificates signed by this CA."""
+        if not self.issuer_url and not self.ocsp_url:
+            return None
+
+        descriptions = []
+        if self.issuer_url:
+            descriptions.append(
+                x509.AccessDescription(
+                    access_method=AuthorityInformationAccessOID.CA_ISSUERS,
+                    access_location=parse_general_name(self.issuer_url),
+                )
+            )
+        if self.ocsp_url:
+            descriptions.append(
+                x509.AccessDescription(
+                    access_method=AuthorityInformationAccessOID.OCSP,
+                    access_location=parse_general_name(self.ocsp_url),
+                )
+            )
+
+        value = x509.AuthorityInformationAccess(descriptions=descriptions)
+        return x509.Extension(oid=ExtensionOID.AUTHORITY_INFORMATION_ACCESS, critical=False, value=value)
+
     def get_authority_key_identifier(self) -> x509.AuthorityKeyIdentifier:
-        """Return the AuthorityKeyIdentifier extension used in certificates signed by this CA.
-
-        Returns
-        -------
-
-        :py:class:`~cg:cryptography.x509.AuthorityKeyIdentifier`
-            The value to use for this extension.
-        """
-
+        """Return the AuthorityKeyIdentifier extension used in certificates signed by this CA."""
         try:
             ski = self.pub.loaded.extensions.get_extension_for_class(x509.SubjectKeyIdentifier)
         except x509.ExtensionNotFound as ex:
             public_key = self.pub.loaded.public_key()
-            if isinstance(public_key, (x448.X448PublicKey, x25519.X25519PublicKey)):  # pragma: no cover
+            if isinstance(public_key, _UNSUPPORTED_PRIVATE_KEY_TYPES):  # pragma: no cover
                 # COVERAGE NOTE: This does not happen in reality, we never generate keys of this type
-                raise TypeError("Cannot get AuthorityKeyIdentifier for X25519 or X448 keys.") from ex
-            return x509.AuthorityKeyIdentifier.from_issuer_public_key(public_key)
+                raise TypeError("Cannot get AuthorityKeyIdentifier from this private key type.") from ex
+            # TYPE NOTE: mypy does not currently recognize isinstance() check above
+            return x509.AuthorityKeyIdentifier.from_issuer_public_key(public_key)  # type: ignore[arg-type]
         else:
             return x509.AuthorityKeyIdentifier.from_issuer_subject_key_identifier(ski.value)
 
-    def get_authority_key_identifier_extension(self) -> AuthorityKeyIdentifier:
-        """Get the AuthorityKeyIdentifier extension to use in certificates signed by this CA.
+    def get_authority_key_identifier_extension(self) -> x509.Extension[x509.AuthorityKeyIdentifier]:
+        """Get the AuthorityKeyIdentifier extension to use in certificates signed by this CA."""
 
-        Returns
-        -------
-
-        :py:class:`~django_ca.extensions.AuthorityKeyIdentifier`
-            The extension to use.
-        """
-
-        return AuthorityKeyIdentifier(
-            x509.Extension(
-                critical=AuthorityKeyIdentifier.default_critical,
-                oid=AuthorityKeyIdentifier.oid,
-                value=self.get_authority_key_identifier(),
-            )
+        return x509.Extension(
+            critical=False,
+            oid=ExtensionOID.AUTHORITY_KEY_IDENTIFIER,
+            value=self.get_authority_key_identifier(),
         )
 
     def get_crl_certs(
@@ -1204,7 +1227,7 @@ class CertificateAuthority(X509CertMixin):
         self.crl_number = json.dumps(crl_number_data)
         self.save()
 
-        return builder.sign(private_key=self.key(password), algorithm=algorithm, backend=default_backend())
+        return builder.sign(private_key=self.key(password), algorithm=algorithm)
 
     def get_password(self) -> Optional[str]:
         """Get password for the private key from the ``CA_PASSWORDS`` setting."""
@@ -1352,7 +1375,7 @@ class AcmeAccount(DjangoCAModel):
         (STATUS_REVOKED, _("Revoked")),
     )
 
-    objects = AcmeAccountManager.from_queryset(AcmeAccountQuerySet)()
+    objects: AcmeAccountManager = AcmeAccountManager.from_queryset(AcmeAccountQuerySet)()
 
     # Account meta data
     created = models.DateTimeField(default=timezone.now)
@@ -1729,7 +1752,7 @@ class AcmeChallenge(DjangoCAModel):
     @property
     def encoded_token(self) -> bytes:
         """Token in base64url encoded form."""
-        return jose.b64encode(self.token.encode("ascii"))
+        return jose.b64.b64encode(self.token.encode("ascii"))
 
     @property
     def expected(self) -> bytes:
@@ -1740,7 +1763,7 @@ class AcmeChallenge(DjangoCAModel):
         if self.type == AcmeChallenge.TYPE_HTTP_01:
             return value
         if self.type == AcmeChallenge.TYPE_DNS_01:
-            return jose.b64encode(hashlib.sha256(value).digest())
+            return jose.b64.b64encode(hashlib.sha256(value).digest())
         raise ValueError(f"{self.type}: Unsupported challenge type.")
 
     def get_challenge(self, request: HttpRequest) -> "messages.ChallengeBody":
@@ -1801,7 +1824,7 @@ class AcmeCertificate(DjangoCAModel):
         :py:class:`~cg:cryptography.x509.CertificateSigningRequest`
             The CSR as used by cryptography.
         """
-        return x509.load_pem_x509_csr(self.csr.encode(), default_backend())
+        return x509.load_pem_x509_csr(self.csr.encode())
 
     @property
     def usable(self) -> bool:
