@@ -14,8 +14,8 @@
 """Module for handling certificate profiles."""
 
 import typing
-import warnings
 from copy import deepcopy
+from datetime import datetime
 from datetime import timedelta
 from threading import local
 from typing import TYPE_CHECKING
@@ -27,11 +27,9 @@ from typing import Union
 from typing import cast
 
 from cryptography import x509
-from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives.hashes import HashAlgorithm
 
 from . import ca_settings
-from .deprecation import RemovedInDjangoCA121Warning
 from .extensions import KEY_TO_EXTENSION
 from .extensions import AuthorityInformationAccess
 from .extensions import AuthorityKeyIdentifier
@@ -79,7 +77,6 @@ class Profile:
         extensions: Optional[Dict[str, Any]] = None,
         cn_in_san: bool = True,
         expires: Optional[Union[int, timedelta]] = None,
-        issuer_name: Optional[Subject] = None,
         description: str = "",
         autogenerated: bool = False,
         add_crl_url: bool = True,
@@ -103,9 +100,6 @@ class Profile:
         else:
             self.subject = Subject(ca_settings.CA_DEFAULT_SUBJECT)
 
-        if issuer_name is not None:
-            warnings.warn("issuer_name profile parameter is deprecated", category=RemovedInDjangoCA121Warning)
-
         self.algorithm = parse_hash_algorithm(algorithm)
         self.cn_in_san = cn_in_san
         self.expires = expires or ca_settings.CA_DEFAULT_EXPIRES
@@ -291,13 +285,11 @@ class Profile:
             subject = Subject(subject)  # NOTE: also accepts None
         cert_subject.update(subject)
 
-        if expires is None:
-            expires = self.expires
         if algorithm is None:
             algorithm = self.algorithm
 
         # Make sure that expires is a fixed timestamp
-        expires = parse_expires(expires)
+        expires = self.get_expires(expires)
 
         # Finally, update SAN with the current CN, if set and requested
         self._update_san_from_cn(cn_in_san, subject=cert_subject, extensions=cert_extensions)
@@ -334,12 +326,18 @@ class Profile:
                 x509.SubjectKeyIdentifier.from_public_key(public_key), critical=False
             )
 
-        return builder.sign(private_key=ca.key(password), algorithm=algorithm, backend=default_backend())
+        return builder.sign(private_key=ca.key(password), algorithm=algorithm)
 
     def copy(self) -> "Profile":
         """Create a deep copy of a profile."""
         return deepcopy(self)
 
+    def get_expires(self, expires: Expires) -> datetime:
+        """Get expiry for the given expiry timestamp."""
+        if expires is None:
+            expires = self.expires
+        return parse_expires(expires)
+
     def serialize(self) -> SerializedProfile:
         """Function to serialize a profile.
 
@@ -372,7 +370,9 @@ class Profile:
         * Adds an IssuerAlternativeName if add_issuer_alternative_name is True
 
         """
-        extensions.setdefault(AuthorityKeyIdentifier.key, ca.get_authority_key_identifier_extension())
+        extensions.setdefault(
+            AuthorityKeyIdentifier.key, AuthorityKeyIdentifier(ca.get_authority_key_identifier_extension())
+        )
 
         if add_crl_url is not False and ca.crl_url:
             crldp = extensions.setdefault(CRLDistributionPoints.key, CRLDistributionPoints())
