@@ -38,13 +38,13 @@ import idna
 
 import asn1crypto.core
 from cryptography import x509
-from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import hashes
 from cryptography.hazmat.primitives.asymmetric import dsa
 from cryptography.hazmat.primitives.asymmetric import ec
 from cryptography.hazmat.primitives.asymmetric import ed448
 from cryptography.hazmat.primitives.asymmetric import ed25519
 from cryptography.hazmat.primitives.asymmetric import rsa
+from cryptography.hazmat.primitives.asymmetric.types import PRIVATE_KEY_TYPES
 from cryptography.hazmat.primitives.serialization import Encoding
 from cryptography.x509.oid import NameOID
 
@@ -55,14 +55,13 @@ from django.utils.translation import gettext_lazy as _
 
 from . import ca_settings
 from .deprecation import RemovedInDjangoCA122Warning
-from .typehints import PRIVATE_KEY_TYPES
 from .typehints import Expires
 from .typehints import Literal
 from .typehints import ParsableGeneralName
 from .typehints import ParsableGeneralNameList
 from .typehints import ParsableHash
-from .typehints import ParsableKeyCurve
 from .typehints import ParsableKeyType
+from .typehints import ParsableName
 from .typehints import SupportsIndex
 
 # List of possible subject fields, in order
@@ -194,33 +193,30 @@ HASH_ALGORITHM_NAMES: typing.Dict[str, typing.Type[hashes.HashAlgorithm]] = {
     hashes.MD5.name: hashes.MD5,
     # hashes.BLAKE2b.name: hashes.BLAKE2b,
     # hashes.BLAKE2s.name: hashes.BLAKE2s,
-    # NOTE: SM3 is added in cryptography 35.0
-    # hashes.SM3.name: hashes.SM3,
+    hashes.SM3.name: hashes.SM3,
 }
-if hasattr(hashes, "SM3"):  # pragma: cryptography>=35.0 branch
-    HASH_ALGORITHM_NAMES[hashes.SM3.name] = hashes.SM3
 
-#: Mapping of canonical elliptic curve names to the implementing classes
+#: Mapping of canonical elliptic curve names (lower-cased) to the implementing classes
 ELLIPTIC_CURVE_NAMES: typing.Dict[str, typing.Type[ec.EllipticCurve]] = {
-    ec.SECT571R1.name: ec.SECT571R1,
-    ec.SECT409R1.name: ec.SECT409R1,
-    ec.SECT283R1.name: ec.SECT283R1,
-    ec.SECT233R1.name: ec.SECT233R1,
-    ec.SECT163R2.name: ec.SECT163R2,
-    ec.SECT571K1.name: ec.SECT571K1,
-    ec.SECT409K1.name: ec.SECT409K1,
-    ec.SECT283K1.name: ec.SECT283K1,
-    ec.SECT233K1.name: ec.SECT233K1,
-    ec.SECT163K1.name: ec.SECT163K1,
-    ec.SECP521R1.name: ec.SECP521R1,
-    ec.SECP384R1.name: ec.SECP384R1,
-    ec.SECP256R1.name: ec.SECP256R1,
-    ec.SECP256K1.name: ec.SECP256K1,
-    ec.SECP224R1.name: ec.SECP224R1,
-    ec.SECP192R1.name: ec.SECP192R1,
-    ec.BrainpoolP256R1.name: ec.BrainpoolP256R1,
-    ec.BrainpoolP384R1.name: ec.BrainpoolP384R1,
-    ec.BrainpoolP512R1.name: ec.BrainpoolP512R1,
+    ec.SECT571R1.name.lower(): ec.SECT571R1,
+    ec.SECT409R1.name.lower(): ec.SECT409R1,
+    ec.SECT283R1.name.lower(): ec.SECT283R1,
+    ec.SECT233R1.name.lower(): ec.SECT233R1,
+    ec.SECT163R2.name.lower(): ec.SECT163R2,
+    ec.SECT571K1.name.lower(): ec.SECT571K1,
+    ec.SECT409K1.name.lower(): ec.SECT409K1,
+    ec.SECT283K1.name.lower(): ec.SECT283K1,
+    ec.SECT233K1.name.lower(): ec.SECT233K1,
+    ec.SECT163K1.name.lower(): ec.SECT163K1,
+    ec.SECP521R1.name.lower(): ec.SECP521R1,
+    ec.SECP384R1.name.lower(): ec.SECP384R1,
+    ec.SECP256R1.name.lower(): ec.SECP256R1,
+    ec.SECP256K1.name.lower(): ec.SECP256K1,
+    ec.SECP224R1.name.lower(): ec.SECP224R1,
+    ec.SECP192R1.name.lower(): ec.SECP192R1,
+    ec.BrainpoolP256R1.name.lower(): ec.BrainpoolP256R1,
+    ec.BrainpoolP384R1.name.lower(): ec.BrainpoolP384R1,
+    ec.BrainpoolP512R1.name.lower(): ec.BrainpoolP512R1,
 }
 
 
@@ -314,8 +310,7 @@ def format_name(subject: typing.Union[x509.Name, x509.RelativeDistinguishedName]
 
     .. deprecated:: 1.20.0
 
-       Passing a list of two-tuples is deprecated as of 1.20.0 and the functionality will be removed in
-       ``django_ca==1.22``.
+       Passing a list of two-tuples is deprecated. The functionality will be removed in ``django_ca==1.22``.
 
     Examples::
 
@@ -339,7 +334,7 @@ def format_name(subject: typing.Union[x509.Name, x509.RelativeDistinguishedName]
         )
         items = subject
 
-    values = "/".join([f"{k}={_format_value(v)}" for k, v in items])
+    values = "/".join([f"{k}={_format_value(v)}" for k, v in items])  # type: ignore[arg-type]
     return f"/{values}"
 
 
@@ -348,8 +343,7 @@ def format_relative_name(name: x509.RelativeDistinguishedName) -> str:
 
     .. deprecated:: 1.20.0
 
-       This function is deprecated in favor of ``format_name()``, which provides identical functionality. This
-       function will be removed in ``django_ca==1.22``.
+       Use ``format_name()`` instead. This function will be removed in ``django_ca==1.22``.
     """
     warnings.warn(
         "This function is deprecated, use format_name() instead.",
@@ -496,7 +490,7 @@ def sanitize_serial(value: str) -> str:
     return serial
 
 
-def parse_name_x509(name: str) -> List[x509.NameAttribute]:
+def parse_name_x509(name: ParsableName) -> typing.Tuple[x509.NameAttribute, ...]:
     """Parses a subject string as used in OpenSSLs command line utilities.
 
     .. versionchanged:: 1.20.0
@@ -509,18 +503,18 @@ def parse_name_x509(name: str) -> List[x509.NameAttribute]:
     whitespace at the start and end is stripped and the subject does not have to start with a slash (``/``).
 
     >>> parse_name_x509('/CN=example.com')
-    [<NameAttribute(oid=<ObjectIdentifier(oid=2.5.4.3, name=commonName)>, value='example.com')>]
+    (<NameAttribute(oid=<ObjectIdentifier(oid=2.5.4.3, name=commonName)>, value='example.com')>,)
     >>> parse_name_x509('c=AT/l= Vienna/o="quoting/works"/CN=example.com')  # doctest: +NORMALIZE_WHITESPACE
-    [<NameAttribute(oid=<ObjectIdentifier(oid=2.5.4.6, name=countryName)>, value='AT')>,
+    (<NameAttribute(oid=<ObjectIdentifier(oid=2.5.4.6, name=countryName)>, value='AT')>,
      <NameAttribute(oid=<ObjectIdentifier(oid=2.5.4.7, name=localityName)>, value='Vienna')>,
      <NameAttribute(oid=<ObjectIdentifier(oid=2.5.4.10, name=organizationName)>, value='quoting/works')>,
-     <NameAttribute(oid=<ObjectIdentifier(oid=2.5.4.3, name=commonName)>, value='example.com')>]
+     <NameAttribute(oid=<ObjectIdentifier(oid=2.5.4.3, name=commonName)>, value='example.com')>)
 
     The function also handles whitespace, quoting and slashes correctly:
 
     >>> parse_name_x509('L="Vienna / District"/CN=example.com')  # doctest: +NORMALIZE_WHITESPACE
-    [<NameAttribute(oid=<ObjectIdentifier(oid=2.5.4.7, name=localityName)>, value='Vienna / District')>,
-     <NameAttribute(oid=<ObjectIdentifier(oid=2.5.4.3, name=commonName)>, value='example.com')>]
+    (<NameAttribute(oid=<ObjectIdentifier(oid=2.5.4.7, name=localityName)>, value='Vienna / District')>,
+     <NameAttribute(oid=<ObjectIdentifier(oid=2.5.4.3, name=commonName)>, value='example.com')>)
 
     Examples of where this string is used are:
 
@@ -530,14 +524,17 @@ def parse_name_x509(name: str) -> List[x509.NameAttribute]:
         # openssl x509 -in cert.pem -noout -subject -nameopt compat
         /C=AT/L=Vienna/CN=example.com
     """
-    attrs = [t.split("=", 1) for t in split_str(name.strip(), "/")]
+
+    if isinstance(name, str):
+        # TYPE NOTE: mypy detects t.split() as Tuple[str, ...] and does not recognize the maxsplit parameter
+        name = tuple(tuple(t.split("=", 1)) for t in split_str(name.strip(), "/"))  # type: ignore[misc]
 
     try:
-        items = [(NAME_CASE_MAPPINGS[t[0].strip().upper()], t[1].strip()) for t in attrs]
+        items = tuple((NAME_CASE_MAPPINGS[t[0].strip().upper()], t[1].strip()) for t in name)
     except KeyError as e:
         raise ValueError(f"Unknown x509 name field: {e.args[0]}") from e
 
-    return [x509.NameAttribute(oid, value) for oid, value in items]
+    return tuple(x509.NameAttribute(oid, value) for oid, value in items)
 
 
 def parse_name(name: str) -> List[Tuple[str, str]]:
@@ -545,8 +542,8 @@ def parse_name(name: str) -> List[Tuple[str, str]]:
 
     .. deprecated:: 1.20.0
 
-       This function has been renamed to :py:func:`~django_ca.utils.parse_name_x509`. The old name will be
-       removed in ``django_ca==1.22``.
+       This function was renamed to :py:func:`~django_ca.utils.parse_name_x509`. The old name will be removed
+       in ``django_ca==1.22``.
     """
 
     warnings.warn(
@@ -557,32 +554,19 @@ def parse_name(name: str) -> List[Tuple[str, str]]:
     attrs = parse_name_x509(name)
 
     # Parse OIDs back to their cannonical string representation
-    return [(OID_NAME_MAPPINGS[attr.oid], attr.value) for attr in attrs]
+    return [(OID_NAME_MAPPINGS[attr.oid], attr.value) for attr in attrs]  # type: ignore[misc]
 
 
-def x509_name(name: str) -> x509.Name:
+def x509_name(name: ParsableName) -> x509.Name:
     """Parses a string into a :py:class:`x509.Name <cg:cryptography.x509.Name>`.
 
-    .. deprecated:: 1.20.0
-
-       Passing a list of two-tuples is deprecated as of 1.20.0 and the functionality will be removed in
-       ``django_ca==1.22``.
-
     >>> x509_name('/C=AT/CN=example.com')
     <Name(C=AT,CN=example.com)>
     """
-    if not isinstance(name, str):
-        warnings.warn(
-            "Passing a list to x509_name() is deprecated, pass a str instead",
-            category=RemovedInDjangoCA122Warning,
-            stacklevel=1,
-        )
-        return x509.Name([x509.NameAttribute(NAME_OID_MAPPINGS[typ], value) for typ, value in name])
-
     return check_name(x509.Name(parse_name_x509(name)))
 
 
-def x509_relative_name(name: str) -> x509.RelativeDistinguishedName:
+def x509_relative_name(name: ParsableName) -> x509.RelativeDistinguishedName:
     """Parse a relative name (RDN) into a :py:class:`~cg:cryptography.x509.RelativeDistinguishedName`.
 
     >>> x509_relative_name('/CN=example.com')
@@ -594,11 +578,6 @@ def x509_relative_name(name: str) -> x509.RelativeDistinguishedName:
     if isinstance(name, x509.RelativeDistinguishedName):
         warnings.warn(msg, category=RemovedInDjangoCA122Warning, stacklevel=1)
         return name
-    if isinstance(name, (list, tuple)):
-        warnings.warn(msg, category=RemovedInDjangoCA122Warning, stacklevel=1)
-        return x509.RelativeDistinguishedName(
-            [x509.NameAttribute(NAME_OID_MAPPINGS[typ], value) for typ, value in name]
-        )
 
     return x509.RelativeDistinguishedName(parse_name_x509(name))
 
@@ -773,12 +752,12 @@ def generate_private_key(
         if key_size is None:
             key_size = ca_settings.CA_DEFAULT_KEY_SIZE
 
-        return dsa.generate_private_key(key_size=key_size, backend=default_backend())
+        return dsa.generate_private_key(key_size=key_size)
     if key_type == "ECC":
         if ecc_curve is None:
             ecc_curve = ca_settings.CA_DEFAULT_ECC_CURVE()
 
-        return ec.generate_private_key(ecc_curve, default_backend())
+        return ec.generate_private_key(ecc_curve)
     if key_type == "EdDSA":
         return ed25519.Ed25519PrivateKey.generate()
     if key_type == "Ed448":
@@ -787,7 +766,7 @@ def generate_private_key(
         if key_size is None:
             key_size = ca_settings.CA_DEFAULT_KEY_SIZE
 
-        return rsa.generate_private_key(public_exponent=65537, key_size=key_size, backend=default_backend())
+        return rsa.generate_private_key(public_exponent=65537, key_size=key_size)
 
     # COVERAGE NOTE: Unreachable code, as all possible key_types are handled above and validate_key_parameters
     #                 would raise for any other key types.
@@ -1091,18 +1070,10 @@ def parse_expires(expires: Expires = None) -> datetime:
     return now + ca_settings.CA_DEFAULT_EXPIRES
 
 
-def parse_key_curve(value: ParsableKeyCurve = None) -> ec.EllipticCurve:
-    """Parse an elliptic curve value.
+def parse_key_curve(value: str) -> ec.EllipticCurve:
+    """Parse a string an :py:class:`~cg:cryptography.hazmat.primitives.asymmetric.ec.EllipticCurve` instance.
 
-    This function uses a value identifying an elliptic curve to return an
-    :py:class:`~cg:cryptography.hazmat.primitives.asymmetric.ec.EllipticCurve` instance. The name must match a
-    class name of one of the classes named under "Elliptic Curves" in
-    :any:`cg:hazmat/primitives/asymmetric/ec`.
-
-    For convenience, passing ``None`` will return the value of :ref:`CA_DEFAULT_ECC_CURVE
-    <settings-ca-default-ecc-curve>`, and passing an
-    :py:class:`~cg:cryptography.hazmat.primitives.asymmetric.ec.EllipticCurve` will return that instance
-    unchanged.
+    This function is intended to parse user input, so it ignores case.
 
     Example usage::
 
@@ -1110,16 +1081,14 @@ def parse_key_curve(value: ParsableKeyCurve = None) -> ec.EllipticCurve:
         <cryptography.hazmat.primitives.asymmetric.ec.SECP256R1 object at ...>
         >>> parse_key_curve('SECP384R1')  # doctest: +ELLIPSIS
         <cryptography.hazmat.primitives.asymmetric.ec.SECP384R1 object at ...>
-        >>> parse_key_curve(ec.SECP256R1())  # doctest: +ELLIPSIS
-        <cryptography.hazmat.primitives.asymmetric.ec.SECP256R1 object at ...>
-        >>> parse_key_curve()  # doctest: +ELLIPSIS
-        <cryptography.hazmat.primitives.asymmetric.ec.SECP256R1 object at ...>
+        >>> parse_key_curve('secp384r1')  # doctest: +ELLIPSIS
+        <cryptography.hazmat.primitives.asymmetric.ec.SECP384R1 object at ...>
 
     Parameters
     ----------
 
-    value : str, otional
-        The name of the curve or ``None`` to return the default curve.
+    value : str
+        The name of the curve (case insensitive).
 
     Returns
     -------
@@ -1133,23 +1102,10 @@ def parse_key_curve(value: ParsableKeyCurve = None) -> ec.EllipticCurve:
     ValueError
         If the named curve is not supported.
     """
-    if isinstance(value, ec.EllipticCurve):
-        return value  # name was already parsed
-    if value is None:
-        return ca_settings.CA_DEFAULT_ECC_CURVE()
-
-    if value in ELLIPTIC_CURVE_NAMES:
-        return ELLIPTIC_CURVE_NAMES[value]()
-
-    error_msg = f"{value}: Not a known Eliptic Curve"
-
     try:
-        curve: Type[ec.EllipticCurve] = getattr(ec, value.strip())
-    except AttributeError as ex:
-        raise ValueError(error_msg) from ex
-    if not issubclass(curve, ec.EllipticCurve):
-        raise ValueError(error_msg)
-    return curve()
+        return ELLIPTIC_CURVE_NAMES[value.strip().lower()]()
+    except KeyError as ex:
+        raise ValueError(f"{value}: Not a known Eliptic Curve") from ex
 
 
 def get_cert_builder(expires: datetime, serial: Optional[int] = None) -> x509.CertificateBuilder:
